% BLP-2LASSO for Aggregate Discrete Choice Models with Rich Covariates
% Authors:  Ben Gillen, Sergio Montero, Roger Moon, & Matt Shum
% Accepted at Econometrics Journal
% Replication Code for Table 4. Results: BLP-2LASSO model
% Last Edited:  April 23, 2019
% Last Run:  April 23, 2019
% Matlab Version - R2017b
%
% Main Procedure for Generating Results appearing in Table 4.  
%
%Outputs:  
%  MPECResults - (K+2 x 4) Summary Table of results with one row for each 
%   variable and columns:
%   - Col1 - Estimated Parameters
%   - Col2 - Estimated Standard Errors
%   - Col3 - tStats
%   - Col4 - p-Value for Coefficient Significance 
%
%Generates Workspaces:
%   Algorithm_5_2_II.mat - Intermediate state after completing Step II
%   Algorithm_5_2_V.mat - Intermediate state after completing Step V
%   Algorithm_5_2_VI.mat - Intermediate state after completing Step VI
%   Algorithm_5_2_VII.mat - Intermediate state after completing Step VII
%   Algorithm_5_2_VIII.mat - Intermediate state after completing Step VIII
%
%
%Calls Functions:
% genXSel
% jacobian
% SelectControls
% Algorithm1_InitialControls
% estimate_BLP


%% Data Preparation

% Clear Memory and Load Source Data
clearvars global 
clear;
clc;
addpath('utilities')
addpath('workspaces')
diary 'workspaces\Table4_Diary.log'

% Load Source Data
disp(['Loading Data:  ' datestr(now)])
load('data160826.mat')

%Identify Markets based on number of Competing Parties
d0=find(electoral12.candCXM==0); N0=size(d0,1); %Five Candidates
d1=find(electoral12.candCXM>0); N1=size(d1,1); %Four Candidates 
dall=find(electoral12.candCXM>=0);
N = N0 + N1;

MarketIndex = [(1:N)';(1:N)';(1:N)';d0;d0;d1];

% Generate Matrix of Covariates (X) 
[AllX, XLabels] = genXSel(electoral12, campaign12, districtChar);
XInD = Algorithm1_InitialControls();
X = AllX(:, XInD);

% Create Global variables for BLP Estimation Program
global share W PX x IV expmeanval 
global sharesum starts 
global T prods JT nn K v oo delta 
global prodsMarket marketStarts marketEnds 

MPECResults_II = estimate_BLP(X, 5, 'workspaces\Algorithm_5_2_II.mat');

% Save Output Workspace(Useful intermediate point)
disp(['Algorithm 5.2.II Complete:  ' datestr(now())])

%% Select Controls for Instruments identifying Variance Components

% Compute Instruments for Variance Components
thetaMPEC2 = MPECResults_II(end-1:end, 1);
Ddelta =  jacobian(thetaMPEC2);       % jacobian matrix of mean utilities

% Select Excluded Controls on Instruments
XIn1 = SelectControls(Ddelta(:, 1), AllX);
XIn2 = SelectControls(Ddelta(:, 2), AllX);
sum((XIn1+XIn2)>0);
NewIn = ((XInD + XIn1 + XIn2)>0) - XInD;

save('workspaces\Algorithm_5_2_V.mat')
disp(['Algorithm 5.2.V Complete:  ' datestr(now())])

%% Add New Controls and Re-Estimate Model
% Add New Controls 
if sum(NewIn) > 0
    AllIn = (XInD + XIn1 + XIn2)>0;
    XInD = AllIn;
end

% Re-Run GMM Optimization
MPECResults_VI = estimate_BLP(AllX(:, XInD), 5, 'workspaces\Algorithm_5_2_VI.mat');
disp(['Algorithm 5.2.VI Complete:  ' datestr(now())])


%% Step VII - Check First Order Conditions for Excluded Controls
load('workspaces\Algorithm_5_2_VI.mat')

% Check FOC's for Remaining Excluded Controls
AllK = size(AllX, 2);
K = K+1;
XOrig = x;
IVOrig = IV;

epsydhatsq = diag(x*covMPEC(1:end-2, 1:end-2)*x');

%Calculate Numerical Derivatives for Each Marginal Variable
step = 0.00001;

AllGrads = zeros(size(AllX, 2), 1);
AllUpsilons = zeros(size(AllX, 2), 2);

for ii = 1:size(AllX, 2)
    iiX = AllX(:, ii);
    iiX = iiX - XOrig*(XOrig\iiX);
    if std(iiX)>0.01
        newX = zscore(iiX);

        Residminus = resid - step*newX;
        qminus = [IVOrig, newX]'*Residminus;
        fminus = qminus(1:end-1)'*W*qminus(1:end-1) + qminus(end)/(newX'*diag(Residminus.^2)*newX);
        Upsilonminus = sqrt(mean((newX.^2).*repmat((Residminus.^2+epsydhatsq), 1, size(newX, 2))));

        Residplus = resid + step*newX;
        qplus = [IVOrig, newX]'*Residplus;
        fplus = qplus(1:end-1)'*W*qminus(1:end-1) + qplus(end)/(newX'*diag(Residplus.^2)*newX);
        Upsilonplus = sqrt(mean((newX.^2).*repmat((Residplus.^2+epsydhatsq), 1, size(newX, 2))));

        fgrad = abs(fplus-fminus)/(2*step*JT);
        AllGrads(ii) = fgrad;
        AllUpsilons(ii, :) = [Upsilonminus(end), Upsilonplus(end)];
    end
end

%Calculate Thresholds and Identify New Variables to Include
gamma = 0.05/log(min(AllK, JT));
c = 1.1;
lambdaQ = c*sqrt(JT*(L-K-2))*norminv(1-gamma/(2*min(AllK, JT)));
    
Thresholds = mean(AllUpsilons, 2)*lambdaQ;
sum(AllGrads > Thresholds);

NewVars = AllGrads > Thresholds;
save('workspaces\Algorithm_5_2_VII.mat')
disp(['Algorithm 5.2.VII Complete:  ' datestr(now())])

%% Step VII: Re-Run GMM Optimization with Complete Set of Selected Controls
clearvars -except NewVars
load('workspaces\Algorithm_5_2_V.mat')
XInD = (XInD + NewVars)>0;
XInD(1450)=0; %Population Density is assumed to be an Excluded Instrument
MPECResults_VIII = estimate_BLP(AllX(:, XInD), 10, 'workspaces\Algorithm_5_2_VIII.mat');
disp(['Algorithm 5.2.VIII Complete:  ' datestr(now())])

%% Calculate Results, Compute SE's using Sandwich Covariance Matrix
load('workspaces\Algorithm_5_2_VIII.mat')

covg = zeros(size(IV,2));
for ii =1:JT
    covg = covg + IV(ii,:)'*IV(ii,:)*(resid(ii)^2);
end
Ddelta = jacobian(thetaMPEC2);       % jacobian matrix of mean utilities
Dg = [x Ddelta]'*IV;            % gradients of moment conditions
covMPEC = ( Dg*W*Dg')\Dg*W*covg*W*Dg'/( Dg*W*Dg');
SEs = sqrt(diag(covMPEC));
tStats = [thetaMPEC1; thetaMPEC2]./sqrt(diag(covMPEC));
pVals = 2*(1-normcdf(abs(tStats)));

MPECResults = [[thetaMPEC1; thetaMPEC2] SEs tStats pVals]

disp(['Estimation Complete:  ' datestr(now)])
disp(MPECResults(1:2, :))
disp(MPECResults(end-1:end, :))

SelectedVars = XLabels(XInD);

save('workspaces\Table4_Results.mat')

diary off


