function XInD = Algorithm1_InitialControls()

% BLP-2LASSO for Aggregate Discrete Choice Models with Rich Covariates
% Authors:  Ben Gillen, Sergio Montero, Roger Moon, & Matt Shum
% Accepted at Econometrics Journal
% Replication Code for Algorithm 1 - Selecting Initial Set of Controls
% Last Edited:  April 23, 2019
% Last Run:  April 23, 2019
% Matlab Version - R2017b
% 
% 
% Function called by Table4_BLP_SelectedControls to initialize set of 
%    Controls for voter preferences based on correlation with log market
%    share subject to L1 penalty.  
%
%Inputs:  Imports and prepares all data
%
%Outputs:  
%  XInD - (K x 1) Vector with entries equal to 1 if the control variable is
%                   selected into the model, 0 otherwise
%
%  Algorithm_5_2_I.mat - Matlab Workspace containing memory state at end of
%      function execution
%
%Calls Functions:  None
% 

%% Data Preparation

% Clear Memory and Load Source Data
clearvars -global 
clear;
clc;

% Load Source Data
disp(['Loading Data:  ' datestr(now)])
load('data160826.mat')

%Identify Markets based on number of Competing Parties
d0=find(electoral12.candCXM==0); N0=size(d0,1); %Five Candidates
d1=find(electoral12.candCXM>0); N1=size(d1,1); %Four Candidates 
dall=find(electoral12.candCXM>=0);
N = N0 + N1;

% Construct Cluster Matrix
dIndex = [dall; dall; dall; d0; d0; d1];
BigD = repmat(dIndex, 1, numel(dIndex))==repmat(dIndex', numel(dIndex), 1);

%Vote Share as Dependent Variable, utside option is casting null vote or abstaining
Y = [log(electoral12.PAN)-log(electoral12.null+electoral12.abstain);... 
    log(electoral12.MP)-log(electoral12.null+electoral12.abstain);...
    log(electoral12.NA)-log(electoral12.null+electoral12.abstain);...
    log(electoral12.PRI(d0))-log(electoral12.null(d0)+electoral12.abstain(d0));...
    log(electoral12.PVEM(d0))-log(electoral12.null(d0)+electoral12.abstain(d0));...
    log(electoral12.CXM(d1))-log(electoral12.null(d1)+electoral12.abstain(d1))];

%Specify Control Variables and Instruments
X = genXSel(electoral12, campaign12, districtChar);
ZHat = genZSimple(electoral12, campaign09, campaign12, districtChar);
Z = [ZHat, X(:, 3:end)];
nZ = size(ZHat, 2); 

%Specify Parameters for Problem
T = size(Y, 1);
KT = size(X, 2);
LT = size(Z, 2);

gamma = 0.05/log(min(KT, T));
c = 1.1;

lambdaX = c*sqrt(T)*norminv(1-gamma/(2*KT))/(2*T);
lambdaX1 = c*sqrt(T)*norminv(1-gamma/(2*(KT-2)))/(2*T);
lambdaX2 = c*sqrt(T)*norminv(1-gamma/(2*(KT-2)))/(2*T);

%Initialization for Upsilon Matrices
UpsilonX = std(X);
UpsilonX1 = std(X(:, 3:end));
UpsilonX2 = std(X(:, 3:end));

xU = X./repmat(UpsilonX, T, 1);
xU1 = X(:, 3:end)./repmat(UpsilonX1, T, 1);
xU2 = X(:, 3:end)./repmat(UpsilonX2, T, 1);

%Iterative Algorithm for Upsilon Matrices, A.1 and A.2
for j = 1:10
    [Bx, StatsX] = lasso(xU, Y, 'Lambda', lambdaX, 'Standardize', false);
    InA = (abs(Bx(3:end))>0.0001);
    Rx1 = xU(:, 1) - xU1(:, InA)*(xU1(:, InA)\xU(:, 1));
    [Bx1, StatsX1] = lasso(xU1, Rx1, 'Lambda', lambdaX1, 'Standardize', false);
    InB = (InA + (abs(Bx1)>0.0001))>0;
    Rx2 = xU(:, 2) - xU2(:, InB)*(xU2(:, InB)\xU(:, 2));
    [Bx2, StatsX2] = lasso(xU2, Rx2, 'Lambda', lambdaX2, 'Standardize', false);

    yResid = Y - xU*Bx;
    UpsilonX = sqrt(mean((X.^2).*(repmat(yResid, 1, KT).^2)));
    xU = X./repmat(UpsilonX, T, 1);

    xU1Resid = xU(:, 1) - xU1*Bx1;
    UpsilonX1 = std(X(:, 3:end).*repmat(xU1Resid, 1, KT-2));
    xU1 = X(:, 3:end)./repmat(UpsilonX1, T, 1);

    xU2Resid = xU(:, 2) - xU2*Bx2;
    UpsilonX2 = std(X(:, 3:end).*repmat(xU2Resid, 1, KT-2));
    xU2 = X(:, 3:end)./repmat(UpsilonX2, T, 1);
end

%Lasso for Selection
[Bx, StatsX] = lasso(xU, Y, 'Lambda', lambdaX, 'Standardize', false);

InA = (abs(Bx(3:end))>0.0001);
Rx1 = xU(:, 1) - xU1(:, InA)*(xU1(:, InA)\xU(:, 1));
[Bx1, StatsX1] = lasso(xU1, Rx1, 'Lambda', lambdaX1, 'Standardize', false);

InB = (InA + (abs(Bx1)>0.0001))>0;
Rx2 = xU(:, 2) - xU2(:, InB)*(xU2(:, InB)\xU(:, 2));
[Bx2, StatsX2] = lasso(xU2, Rx2, 'Lambda', lambdaX2, 'Standardize', false);

XInD = logical([true*ones(11, 1); ((abs(Bx1(10:end))>0.001)+(abs(Bx2(10:end))>0.001)+(abs(Bx(12:end))>0.001))>0]);

%2SLS for Estimation (Results not Reported)
xIn = X(:,XInD);
zIn = [Z(:,1:nZ) X(:, [false false XInD(3:end)'])];

xHat = zIn*(zIn\xIn);
B=xHat\Y;

Epsy = Y - xIn*B;
S=xHat'*((Epsy*Epsy').*(BigD))*xHat; 
VB=(xHat'*xHat)\(S)/(xHat'*xHat); 
seB=sqrt(diag(VB)); 

Results=[B, seB, abs(B./seB), 2*(1-normcdf(abs(B./seB)))];

% Remove raw data to save memory
clear X Z xU xU1 xU2 

save('workspaces\Algorithm_5_2_I.mat')
disp(['Algorithm 5.2.I Complete:  ' datestr(now())])
