function MPECResults = estimate_BLP(X, UpdateCount, strWorkspaceFilename)
% BLP-2LASSO for Aggregate Discrete Choice Models with Rich Covariates
% Authors:  Ben Gillen, Sergio Montero, Roger Moon, & Matt Shum
% Last Edited:  April 23, 2019
% Last Run:  April 23, 2019
% Matlab Version - R2017b
%
% Function called by Table4_BLP_SelectedControls to estimate BLP Model for 
%    Selected Control Variables X
%
%Inputs:  
%  X - (JT x K) Matrix of Controls for Consumer Preferences
%  UpdateCount - Number of Updates to GMM Weighting Matrix
%  strWorkspaceFilename - String specifying filename for final workspace
%
%Outputs:  
%  MPECResults - (K+2 x 4) Summary Table of results with one row for each 
%   variable and columns:
%   - Col1 - Estimated Parameters
%   - Col2 - Estimated Standard Errors
%   - Col3 - tStats
%   - Col4 - p-Value for Coefficient Significance 
%
%  strWorkspace.mat - Matlab Workspace containing memory state at end of
%      function execution
%
%Calls Functions:
% genZSimple
% invertshares
% GMMMPEC_f_ktr
% GMMMPEC_c_ktr
% GMMMPEC_hess_ktr
% jacobian

% Load Source Data
disp(['Loading Data:  ' datestr(now)])
load('data160826.mat')

%Identify Markets based on number of Competing Parties
d0=find(electoral12.candCXM==0); N0=size(d0,1); %Five Candidates
d1=find(electoral12.candCXM>0); N1=size(d1,1); %Four Candidates 
dall=find(electoral12.candCXM>=0);
N = N0 + N1;

MarketIndex = [(1:N)';(1:N)';(1:N)';d0;d0;d1];

%Generate Instruments (Z)
ZHat = genZSimple(electoral12, campaign09, campaign12, districtChar);
Z = [ZHat, X(:, 3:end)];

%Create Global variables for MPEC Optimization Program
global share W PX x IV expmeanval 
global sharesum starts 
global T prods JT nn K v oo delta 
global prodsMarket marketStarts marketEnds 


%Calculate Vote Shares and Abstention Shares
share=[(electoral12.PAN)./(electoral12.registered);...
    (electoral12.MP)./(electoral12.registered);...
    (electoral12.NA)./(electoral12.registered);...
    (electoral12.PRI(d0))./(electoral12.registered(d0));...
    (electoral12.PVEM(d0))./(electoral12.registered(d0));...
    (electoral12.CXM(d1))./(electoral12.registered(d1))];

nopurch=[(electoral12.null+electoral12.abstain)./(electoral12.registered);...
    (electoral12.null+electoral12.abstain)./(electoral12.registered);...
    (electoral12.null+electoral12.abstain)./(electoral12.registered);...
    (electoral12.null(d0)+electoral12.abstain(d0))./(electoral12.registered(d0));...
    (electoral12.null(d0)+electoral12.abstain(d0))./(electoral12.registered(d0));...
    (electoral12.null(d1)+electoral12.abstain(d1))./(electoral12.registered(d1))];

y = log(share) - log(nopurch);

% Populate dimensionality values for Global Variables
JT = size(share, 1);
prods = 7;
T = numel(electoral12.candCXM);
K = size(X, 2);
L = size(Z, 2);
K0 = 2;

% Generate draws for share integrals during estimation
rng(123456)
nn = 2000;
v = randn(2,nn);                     

% Sort Data by District
Reindex = [(1:JT)' [(1:N)'*1000;(1:N)'*1000;(1:N)'*1000;d0*1000;d0*1000;d1*1000]*1000+ ...
                    [1*ones(N, 1); 2*ones(N, 1); 3*ones(N, 1); 4*ones(N0, 1); 5*ones(N0, 1); 6*ones(N1, 1);]];
SortIV = sortrows([Reindex(:, 2) Z], 1);
Sortx =  sortrows([Reindex(:, 2) X], 1);
Sorty =  sortrows([Reindex(:, 2) y], 1);
Sortshare = sortrows([Reindex(:, 2) share], 1);
SortMarketIndex = sortrows([Reindex(:, 2) MarketIndex], 1);

% Populate Data values for Global Variables
IV = SortIV(:, 2:end);
x = Sortx(:, 2:end);
y = Sorty(:, 2:end);
share = Sortshare(:, 2:end);
MarketIndex = SortMarketIndex(:, 2);

% Populate number of products per market & related data features
starts=1;
oo = ones(1,nn);  % index for use in simulation of shares

% Create denominators in logit predicted shares (sums numerators)
sharesum = sparse(zeros(JT,JT));  
for t = 1:JT
    for s = 1:JT
        sharesum(t, s) = (SortMarketIndex(t, 2) == SortMarketIndex(s,2));
    end
end

% District Index and Parties Competing in Each District
marketStarts = zeros(T, 1);
marketEnds = zeros(T, 1);
for t = 1:T
    marketStarts(t) = min(find(MarketIndex==t));
    marketEnds(t) = max(find(MarketIndex==t));
end

prodsMarket = marketEnds - marketStarts+1;


%% MPEC ESTIMATION OF RANDOM COEFFICIENTS LOGIT MODEL 
disp(['Setting Up Optimization Problem:  ' datestr(now)])

% Set Initial Guesses and Identity Weighting Matrix
nIV = size(IV,2);       
expmeanval0 = exp(y);    
W = eye(nIV);


GMPEC = 1.0e20;
CPUtMPEC = 0;
FuncEvalMPEC = 0;
GradEvalMPEC = 0;
HessEvalMPEC = 0;
expmeanval = expmeanval0;
reps=1;

% Set starting values for optimization problem
theta20 = [0.05;0.00001];            % Random Coefficient Variance
delta0 = invertshares(theta20); % Mean Utilities via BLP Inversion
xhat = IV*(IV\x);
theta10 = xhat\delta0;            % Coefficients via 2SLS
xi0 = delta0 - x*theta10;         % Expected Utility Residuals 
g0 = IV'*xi0;                     % GMM loss function

% Consolidate starting values into single vector
x0 = [theta10; theta20; delta0; g0];  

% Set min/max bounds for each of the 
x_L = -Inf*ones(length(x0),1);   % Lower bounds for x.
x_L(K+1:K+2) = 0;                % Non-Negative Random Coefficient Variance
x_U =  Inf*ones(length(x0),1);   % No upper bound for x
x_U(K+1:K+2) = 1;                % Upper Bound on RC Variance

% number of MPEC optimization parameters
nx0 = size(x0,1);   

% Sparsity patterns of Constraint Gradient equals 0 if derivative equals 0
% Derivatives of market shares
c11 = zeros(JT, K); % market shares with respect to mean parameters
c12 = ones(JT, 2);  % market shares with respect to standard deviations
c13 = zeros(JT,JT); % market shares with respect to mean utilities
for t=1:T
    c13(marketStarts(t):marketEnds(t),marketStarts(t):marketEnds(t)) = 1;
end
c14 = zeros(JT, nIV); % market shares with respect to moment values

% Derivatives of moments
c21 = ones(nIV, K);   % moments with respect to mean parameters 
c22 = zeros(nIV, 2);    % moments with respect to standard deviations
c23 = ones(nIV, JT);  % moments with respect to mean utilities
c24 = eye(nIV);       % moments with respect to moment values 

ConsPattern = [ c11 c12 c13 c14; c21 c22 c23 c24 ];   

% Hessian pattern
HessianPattern = zeros(nx0, nx0);
HessianPattern(K+3+JT:end, K+3+JT:end) = ones(nIV, nIV);
HessianPattern(K+1:K+2,K+1:K+2+JT) = ones(2, 2+JT);
HessianPattern(K+3:K+2+JT,K+1:K+2) = ones(JT, 2);
for t=1:T
    range = marketStarts(t):marketEnds(t);
    index = K+2+range;
    HessianPattern(index, index) = ones(prodsMarket(t), prodsMarket(t));
end 

%Set Options for K-Nitro Optimization 
ktropts = optimset('DerivativeCheck','off','Display','iter',...
       'TolCon',1E-8,'TolFun',1E-8,'TolX',1E-8,...
       'GradConstr','on','GradObj','on',...
       'JacobPattern',ConsPattern,...
       'Hessian','user-supplied','HessFcn',@GMMMPEC_hess_ktr,...
       'HessPattern', HessianPattern);  

%Run Optimization for GMM Objective Function
disp(['Optimization Iteration 1:  ' datestr(now)])
myfun = @(x)GMMMPEC_f_ktr(x);
%Code using fmincon
[BetaMPEC fval_rep exitflag output lambda] = ...
    fmincon(myfun,x0,[],[],[],[],x_L,x_U,...
                  @(x)GMMMPEC_c_ktr(x), ktropts);
              
%Collect Parameters from Estimation
theta1MPEC_rep = BetaMPEC(1:K);
theta2MPEC_rep = BetaMPEC(K+1:K+2);
GMPEC_rep = fval_rep;
INFOMPEC_rep = exitflag;

if (GMPEC_rep < GMPEC && (INFOMPEC_rep==0 || INFOMPEC_rep==2 || INFOMPEC_rep==1))
     thetaMPEC1 = theta1MPEC_rep;
     thetaMPEC2 = theta2MPEC_rep;
     GMPEC = GMPEC_rep;
     INFOMPEC = INFOMPEC_rep;
     x0 = BetaMPEC;
end



%% Perform Continuous Updating
for UpdatingIter = 1:UpdateCount
    %Compute Covariance Matrix for Moments and Recompute Weighting Matrix
    deltaSE = invertshares(thetaMPEC2);    
    resid = deltaSE - x*thetaMPEC1;        

    covg = zeros(size(IV,2));
    for ii =1:JT
        covg = covg + IV(ii,:)'*IV(ii,:)*(resid(ii)^2)/JT;
    end
    W = inv(covg);
    %Regularize weighting matrix by reinforcing the diagonal
    W = W + mean(diag(W))*eye(size(W, 1))/(UpdatingIter^2);

    %Reoptimize GMM Objective with Updated Weighting Matrix
    disp(['Begin Optimization Update ' num2str(UpdatingIter) ':  '  datestr(now)])
    [BetaMPEC, fval_rep, exitflag, output, lambda] = ...
        fmincon(myfun,x0,[],[],[],[],x_L,x_U,...
                      @(x)GMMMPEC_c_ktr(x), ktropts);
              
    %Collect Parameters from Estimation
    theta1MPEC_rep = BetaMPEC(1:K);
    theta2MPEC_rep = BetaMPEC(K+1:K+2);
    GMPEC_rep = fval_rep;
    INFOMPEC_rep = exitflag;

    if (INFOMPEC_rep==0 || INFOMPEC_rep==2 || INFOMPEC_rep==1)
         thetaMPEC1 = theta1MPEC_rep;
         thetaMPEC2 = theta2MPEC_rep;
         GMPEC = GMPEC_rep;
         INFOMPEC = INFOMPEC_rep;
         x0 = BetaMPEC;
    end

    %Compute Covariance Matrix for Moments and Recompute Weighting Matrix
    deltaSE = invertshares(thetaMPEC2);    
    resid = deltaSE - x*thetaMPEC1;        


end


%% Compute Standard Errors using Sandwich Covariance Matrix
covg = zeros(size(IV,2));
for ii =1:JT
    covg = covg + IV(ii,:)'*IV(ii,:)*(resid(ii)^2);
end
Ddelta = jacobian(thetaMPEC2);       % jacobian matrix of mean utilities
Dg = [x Ddelta]'*IV;            % gradients of moment conditions
covMPEC = inv( Dg*W*Dg')*Dg*W*covg*W*Dg'*inv( Dg*W*Dg');
SEs = sqrt(diag(covMPEC));
tStats = [thetaMPEC1; thetaMPEC2]./sqrt(diag(covMPEC));
pVals = 2*(1-normcdf(abs(tStats)));

MPECResults = [[thetaMPEC1; thetaMPEC2] SEs tStats pVals]

disp(['Estimation Complete:  ' datestr(now)])
disp(MPECResults(1:2, :))
disp(MPECResults(end-1:end, :))


save(strWorkspaceFilename)