function X = genXCollapse(electoral12, campaign12, districtChar)

% BLP-2LASSO for Aggregate Discrete Choice Models with Rich Covariates
% Authors:  Ben Gillen, Sergio Montero, Roger Moon, & Matt Shum
% Last Edited:  April 23, 2019
% Last Run:  April 23, 2019
% Matlab Version - R2017b
%
% Function called by Table3_BLP_FixedControls to generate set of fixed
%    control variables for estimating BLP Voter Share Model
% 
%Inputs:  
%  electoral12 - (300 x 13) Matrix of Outcomes from 2012 Election
%  campaign12 - (300 x 8) Matrix of Campaign Expenditures in 2012 Election
%  districtChar - (300 x 197) Matrix of Election District Demographics
%
%Output:  
%  X - (1301 x 95) Matrix of Possible Included Control Variables
%      P (1301 x 2) - Expenditures and Expenditures^2
%      XJ (1301 x 9) - Party and Coalition State Fixed Effects
%      X0 (1301 x 14) - Control Variables: with Columns
%         1-4 Region Fixed Effects
%         5   female heads of household (% of total households)
%         6   population aged 18-24 (% of voting-age pop.)
%         7   population over 65 (% of voting-age pop.)
%         8   married (% of pop. over 12)
%         9   pop. with post-elementary education (% of voting-age pop.)
%         10  average years of schooling
%         11  unemployment rate
%         12  households without basic utilities
%         13  households with a fridge
%         14  households with a car
%      X0Star (1301 x 84) - Control Variables interacted with Party 

d0=find(electoral12.candCXM==0); N0=size(d0,1); %Five Candidates
d1=find(electoral12.candCXM>0); N1=size(d1,1); %Four Candidates - Affiliation 1
N = N0 + N1;


P = 10^(-4)*[campaign12.PAN; 
             campaign12.MP;
             campaign12.NA;
             campaign12.PRI(d0); 
             campaign12.PVEM(d0); 
             campaign12.CXM(d1)];
P = [P, P.^2];

PartyMP = zeros(N, 3);
for n = 1:N
    PartyMP(n, electoral12.candMP(n)+1)=1;
end

PartyCX = zeros(N, 3);
for n = 1:N
    PartyCX(n, electoral12.candCXM(n)+1)=1;
end
PartyCX = PartyCX(d1, 2:3);

XJ = blkdiag(ones(N, 1), PartyMP, ones(N, 1), ones(N0, 1), ones(N0, 1), PartyCX);
X0 = zeros(N, 14);
X0(:,1)=districtChar.CIRCUNSCRIPCI__N==1; % northwest
X0(:,2)=districtChar.CIRCUNSCRIPCI__N==2; % northeast
X0(:,3)=districtChar.CIRCUNSCRIPCI__N==3; % southeast
X0(:,4)=districtChar.CIRCUNSCRIPCI__N==4; % southwest (region 5: center)
X0(:,5)=districtChar.HOGJEF_F_N_32_10./districtChar.TOTHOG_N_32_10; % female heads of household (% of total households)
X0(:,6)=districtChar.P_18A24_N_32_10./districtChar.P_18YMAS_N_32_10; % population aged 18-24 (% of voting-age pop.)
X0(:,7)=districtChar.POB65_MAS_N_32_10./districtChar.P_18YMAS_N_32_10; % population over 65 (% of voting-age pop.)
X0(:,8)=districtChar.P12YM_CASA_N_32_10./(districtChar.P12YM_CASA_N_32_10+districtChar.P12YM_SOLT_N_32_10+districtChar.P12YM_SEPA_N_32_10); % married (% of pop. over 12)
X0(:,9)=1-districtChar.P18YM_PB_N_32_10./districtChar.P_18YMAS_N_32_10; % pop. with post-elementary education (% of voting-age pop.)
X0(:,10)=districtChar.GRAPROES_N_32_10; % average years of schooling
X0(:,11)=1-districtChar.POCUPADA_N_32_10./districtChar.PEA_N_32_10; % unemployment rate
X0(:,12)=1-districtChar.VPH_C_SERV_N_32_10./districtChar.VIVPAR_HAB_N_32_10; % households without basic utilities
X0(:,13)=districtChar.VPH_REFRI_N_32_10./districtChar.VIVPAR_HAB_N_32_10; % households with a fridge
X0(:,14)=districtChar.VPH_AUTOM_N_32_10./districtChar.VIVPAR_HAB_N_32_10; % households with a car
ZeroX0 = (std(X0)==0);
X0 = zscore(X0(:, ~ZeroX0));

X0Star = blkdiag(X0, X0, X0, X0(d0,:), X0(d0,:), X0(d1,:));

X = [P XJ X0Star];