function [X, XLabels] = genXSel(electoral12, campaign12, districtChar)

% BLP-2LASSO for Aggregate Discrete Choice Models with Rich Covariates
% Authors:  Ben Gillen, Sergio Montero, Roger Moon, & Matt Shum
% Last Edited:  April 23, 2019
% Last Run:  April 23, 2019
% Matlab Version - R2017b
%
% Function called by Table4_BLP_SelectedControls, estimate_BLP, and  
%   Algorithm1_InitialControls to calculate set of control variables for 
%   estimating BLP Model with Mexico Election Data
% 
%Inputs:  
%  electoral12 - (300 x 13) Matrix of Outcomes from 2012 Election
%  campaign12 - (300 x 8) Matrix of Campaign Expenditures in 2012 Election
%  districtChar - (300 x 197) Matrix of Election District Demographics
%
%Output:  
%  X - (1301 x 3671) Matrix of Possible Included Control Variables, with
%     base variables from genXCollapse and remaining demographic variables
%     from districtChar, including their logs and squares.  
%  XLabels - (3671 x 1) Cell array of labels for each of the X Variables

d0=find(electoral12.candCXM==0); N0=size(d0,1); %Five Candidates
d1=find(electoral12.candCXM>0); N1=size(d1,1); %Four Candidates - Affiliation 1
N = N0 + N1;

P = 10^(-4)*[campaign12.PAN; 
             campaign12.MP;
             campaign12.NA;
             campaign12.PRI(d0); 
             campaign12.PVEM(d0); 
             campaign12.CXM(d1)];
P = [P, P.^2];

PartyMP = zeros(N, 3);
for n = 1:N
    PartyMP(n, electoral12.candMP(n)+1)=1;
end

PartyCX = zeros(N, 3);
for n = 1:N
    PartyCX(n, electoral12.candCXM(n)+1)=1;
end
PartyCX = PartyCX(d1, 2:3);

XJ = blkdiag(ones(N, 1), PartyMP, ones(N, 1), ones(N0, 1), ones(N0, 1), PartyCX);
X0 = zeros(N, 14);
X0(:,1)=districtChar.CIRCUNSCRIPCI__N==1; % northwest
X0(:,2)=districtChar.CIRCUNSCRIPCI__N==2; % northeast
X0(:,3)=districtChar.CIRCUNSCRIPCI__N==3; % southeast
X0(:,4)=districtChar.CIRCUNSCRIPCI__N==4; % southwest (region 5: center)
X0(:,5)=districtChar.HOGJEF_F_N_32_10./districtChar.TOTHOG_N_32_10; % female heads of household (% of total households)
X0(:,6)=districtChar.P_18A24_N_32_10./districtChar.P_18YMAS_N_32_10; % population aged 18-24 (% of voting-age pop.)
X0(:,7)=districtChar.POB65_MAS_N_32_10./districtChar.P_18YMAS_N_32_10; % population over 65 (% of voting-age pop.)
X0(:,8)=districtChar.P12YM_CASA_N_32_10./(districtChar.P12YM_CASA_N_32_10+districtChar.P12YM_SOLT_N_32_10+districtChar.P12YM_SEPA_N_32_10); % married (% of pop. over 12)
X0(:,9)=1-districtChar.P18YM_PB_N_32_10./districtChar.P_18YMAS_N_32_10; % pop. with post-elementary education (% of voting-age pop.)
X0(:,10)=districtChar.GRAPROES_N_32_10; % average years of schooling
X0(:,11)=1-districtChar.POCUPADA_N_32_10./districtChar.PEA_N_32_10; % unemployment rate
X0(:,12)=1-districtChar.VPH_C_SERV_N_32_10./districtChar.VIVPAR_HAB_N_32_10; % households without basic utilities
X0(:,13)=districtChar.VPH_REFRI_N_32_10./districtChar.VIVPAR_HAB_N_32_10; % households with a fridge
X0(:,14)=districtChar.VPH_AUTOM_N_32_10./districtChar.VIVPAR_HAB_N_32_10; % households with a car
ZeroX0 = (std(X0)==0);
X0 = X0(:, ~ZeroX0);

X0Labels = {'Region 1', 'Region 2', 'Region 3', 'Region 4', 'Female HoH', ...
            'Pop 18-24', 'Pop 65+', 'Pop Married', 'Pop Elementary+', ...
            'Yrs School', 'Unemployment', 'Utilities', 'Fridge', 'Car'};
% Include all other District Characteristics in X0, eliminating those with
% zero variability
X0 = [X0 districtChar{:, 4:end}];
X0Labels = {X0Labels{:}, districtChar.Properties.VariableNames{4:end}}; 
ZeroX0 = (std(X0)==0);
X0 = X0(:, ~ZeroX0);
X0Labels = X0Labels(:, ~ZeroX0);

BinaryCheckCount = size(X0, 2);
BinaryCheck = zeros(1, BinaryCheckCount);
for ii = 1:BinaryCheckCount
    UniqueCount = numel(unique(X0(:, ii)));
    BinaryCheck(ii) = (UniqueCount==2);
end

%Include X0, log(X0), and Xo.^2
X0 = zscore([X0 log(X0(:, ~BinaryCheck)+1)  X0(:, ~BinaryCheck).^2]);

LogX0Labels = X0Labels(~BinaryCheck);
for ii = 1:numel(LogX0Labels)
    LogX0Labels{ii} = ['log ' LogX0Labels{ii}];
end
sqX0Labels = X0Labels(~BinaryCheck);
for ii = 1:numel(LogX0Labels)
    LogX0Labels{ii} = ['square ' sqX0Labels{ii}];
end
X0Labels = {X0Labels{:}, LogX0Labels{:},  sqX0Labels{:}};


X0Star = blkdiag(X0, X0, X0, X0(d0,:), X0(d0,:), X0(d1,:));

X0StarLabels1 = X0Labels;
X0StarLabels2 = X0Labels;
X0StarLabels3 = X0Labels;
X0StarLabels4 = X0Labels;
X0StarLabels5 = X0Labels;
X0StarLabels6 = X0Labels;


for ii = 1:numel(X0StarLabels1)
    X0StarLabels1{ii} = [X0Labels{ii} ' PAN' ];
    X0StarLabels2{ii} = [X0Labels{ii} ' MP'];
    X0StarLabels3{ii} = [X0Labels{ii} ' NA'];
    X0StarLabels4{ii} = [X0Labels{ii} ' PRI'];
    X0StarLabels5{ii} = [X0Labels{ii} ' PVEM'];
    X0StarLabels6{ii} = [X0Labels{ii} ' CXM'];
end
X0StarLabels = {X0StarLabels1{:} X0StarLabels2{:} X0StarLabels3{:} X0StarLabels4{:} X0StarLabels5{:} X0StarLabels6{:}};

X = [P XJ X0Star];
XLabels = {'Price', 'PriceSquare', ...
            'PartyFE1', 'PartyFE2', 'PartyFE3', 'PartyFE4', ...
            'PartyFE5', 'PartyFE6', 'PartyFE7', 'PartyFE8', ...
            'PartyFE9', X0StarLabels{:}};


