function Z = genZSimple(electoral12, campaign09, campaign12, ...
                            districtChar)

% BLP-2LASSO for Aggregate Discrete Choice Models with Rich Covariates
% Authors:  Ben Gillen, Sergio Montero, Roger Moon, & Matt Shum
% Last Edited:  April 23, 2019
% Last Run:  April 23, 2019
% Matlab Version - R2017b
%
% Function called by Table3_BLP_FixedControls, estimate_BLP, and  
%   Algorithm1_InitialControls to calculate excluded instruments for 
%   estimating BLP Model with Mexico Election Data
% 
%Inputs:  
%  electoral12 - (300 x 13) Matrix of Outcomes from 2012 Election
%  campaign09 - (300 x 9) Matrix of Campaign Expenditures in 2009 Election
%  campaign12 - (300 x 8) Matrix of Campaign Expenditures in 2012 Election
%  districtChar - (300 x 197) Matrix of Election District Demographics
%
%Output:  
%  Z - (1301 x 8) Matrix of Excluded Instruments 


d0=find(electoral12.candCXM==0); N0=size(d0,1); %Five Candidates
d1=find(electoral12.candCXM>0); N1=size(d1,1); %Four Candidates
N = N0 + N1;

% Lagged-spending instruments
LaggedZ=10^(-4)*[campaign09.PAN;...
                 campaign09.PRD+campaign09.SM;...
                 campaign09.NA;...
                 campaign09.PRI(d0)+0.9*campaign09.PM(d0);...
                 campaign09.PVEM(d0)+0.1*campaign09.PM(d0);...
                 campaign09.PRI(d1)+campaign09.PVEM(d1)+campaign09.PM(d1)];
LaggedZ=[LaggedZ LaggedZ.^2];

% BLP - Average spending by rival parties in neighboring districts
BLP = zeros(N, 6);
for n = 1:N
    for j = 1:6
        nInState = logical((campaign12.state == campaign12.state(n))...
                        .*(campaign12.district ~= campaign12.district(n)));
        zState = zeros(sum(nInState), 1);
        ii = 0;
        for nn = 1:N
            if nInState(nn)==1
                ii=ii+1;
                iiInParties = [false false (campaign12{nn, 3:end}>0)];
                iiInParties(j) = false; 
                zState(ii) = 10^(-4)*mean(campaign12{nn, iiInParties});
            end
        end
        BLP(n,j) = mean(zState);
    end
end
BLPZ = [BLP(:,1);BLP(:,2);BLP(:,3);BLP(d0,4);BLP(d0,5);BLP(d1,6)];
BLPZ = [BLPZ  BLPZ.^2];

% Population Density as cost shifter
Density = districtChar.DensidadDePob18_/1000;
DensityZ =[Density; Density; Density; Density(d0);Density(d0);Density(d1)];
DensityZ =[DensityZ DensityZ.^2];

% Internet Access as cost shifter
Internet = districtChar.VPH_INTER_N_32_10./districtChar.VIVPAR_HAB_N_32_10;
InternetZ=[Internet;Internet;Internet;Internet(d0);Internet(d0);Internet(d1)];
InternetZ=[InternetZ InternetZ.^2];

Z = [LaggedZ BLPZ DensityZ InternetZ];